//
// GameX - Buffer Class Header
// 
// Copyright (C) 2002 Rama C. Hoetzlein
// 
// This software is released under the GameX GNU GPL 
// Open Source License. See the GameX documentation included
// with this source code for terms of modification, 
// distribution and re-release. 
// 

//#include "gamex-win-dx.h"


#ifndef GAMEX_BUFFER_H_INCLUDED
#define GAMEX_BUFFER_H_INCLUDED

#include "gamex-global-def.h"

///	#include <stdio.h>
///	#include <stdlib.h>
///	#include <string.h>
///	#include <conio.h>

///	#include "gamex-debug.h"
///	#include "gamex-defines.h"

#define BUFFER_DEBUG
// #define BUFFER_TESTER

#define FALSE					0
#define TRUE					1

#define BUFFER_ORDER_LBF		0
#define BUFFER_ORDER_MBF		1

#define BUFFER_EMPTY		0
#define BUFFER_DATA			1
#define BUFFER_STRING		2
#define BUFFER_CONSOLE		3
#define BUFFER_ARRAY		4

class Buffer
{
public:	
	Buffer ();										// Initialize a TYPE_EMPTY buffer
	
	void SetOrder (int order);							// Set byte order for interpretation

	void Invert (unsigned char *in, int width);
	void Scale (unsigned char *in, int width, int scale);

	int UnstuffEachBit (unsigned char *in, unsigned char *out, int width);

	int ReadWord (char *line, char *word);				// Read words (space delimited)
	int ReadWord (char *line, char *word, char delim);	// Read words (arbitrary delimited)
														
	// Read Bytes, Words, DWords, Ints, Floats, Doubles according to Byte Order
	inline unsigned char ReadC (unsigned char c) {return (*FuncReadC) (c);}
	inline unsigned short int ReadI (unsigned short c) {return (*FuncReadI) (c);}	
	inline unsigned long int ReadL (unsigned long c) {return (*FuncReadL) (c);}

	// Write Bytes, Words, DWords, Ints, Floats, Doubles according to Byte Order
	inline unsigned char WriteC (unsigned char c) {return (*FuncWriteC) (c);}
	inline unsigned short WriteI (unsigned short int c) {return (*FuncWriteI) (c);}	
	inline unsigned long WriteL (unsigned long int c) {return (*FuncWriteL) (c);}
private:
	int byte_order;
	short int buf_type;
	int rows, cols;
	int curr_row, curr_col;
	char *buf, *curr_pos;
	
	unsigned char (*FuncReadC) (unsigned char c);
	unsigned short int (*FuncReadI) (unsigned short c);
	unsigned long int (*FuncReadL) (unsigned long c);

	unsigned char (*FuncWriteC) (unsigned char c);
	unsigned short (*FuncWriteI) (unsigned short int c);
	unsigned long (*FuncWriteL) (unsigned long int c);
};

#ifndef BUFFER_STATIC
	#define BUFFER_STATIC
	static unsigned char Mask[8] = {128, 64, 32, 16, 8, 4, 2, 1};		
#endif

static inline unsigned char ReadC_LBF (unsigned char c) {return c;}		
static inline unsigned short int ReadI_LBF (unsigned short c) {return c;}		
static inline unsigned long int ReadL_LBF (unsigned long c) {return c;}

static inline unsigned char WriteC_LBF (unsigned char  c) {return c;}		
static inline unsigned short WriteI_LBF (unsigned short int c) {return c;}		
static inline unsigned long WriteL_LBF (unsigned long int c) {return c;}

static inline unsigned char ReadC_MBF (unsigned char c) {return c;}
static inline unsigned char WriteC_MBF (unsigned char c) {return c;}

#define Int2to1(c)		((c & 0xFF00) >> 8)
#define Int1to2(c)		((c & 0xFF) << 8)
#define IntReverse(c)	(Int2to1(c) | Int1to2(c))
static inline unsigned short int ReadI_MBF (unsigned short c)
{
	return (unsigned short int) IntReverse(c);		
}
static inline unsigned short WriteI_MBF (unsigned short int c)
{
	return (unsigned short) IntReverse(c);		
}

#define Long4to1(c)		((c & 0xFF000000) >> 24)
#define Long3to2(c)		((c & 0xFF0000) >> 8)
#define Long2to3(c)		((c & 0xFF00) << 8)
#define Long1to4(c)		((c & 0xFF) << 24)
#define LongReverse(c)	(Long4to1(c) | Long3to2(c) | Long2to3(c) | Long1to4(c))	
static inline unsigned long int ReadL_MBF (unsigned long c)
{
	return (unsigned long int) LongReverse(c);
}	
static inline float ReadF_MBF (unsigned long c)
{
	return (float) LongReverse(c);
}
static inline unsigned long WriteL_MBF (unsigned long int c)
{
	return (unsigned long) LongReverse(c);		
}

#endif