//
// GameX - File Class Header
// 
// Copyright (C) 2002 Rama C. Hoetzlein
// 
// This software is released under the GameX GNU GPL 
// Open Source License. See the GameX documentation included
// with this source code for terms of modification, 
// distribution and re-release. 
// 

//#include "gamex-win-dx.h"


#ifndef GAMEX_FILE_H_INCLUDED
#define GAMEX_FILE_H_INCLUDED

#include "gamex-global-def.h"


//	#include <stdio.h>
//	#include <stdlib.h>
//	#include <conio.h>
//	#include <string.h>

//	#include "gamex-debug.h"			// Access debugging
//	#include "gamex-defines.h"		// Necessary file-related constants

								//**** File Class Switches
#define FILE_DEBUG				// Required for File class
#define FILE_CHECKS				// Extra File operation checks
// #define FILE_COUNT_WRITES		// Counts # of file writes in write_number field

#define FILE_BUFFER			2000
#define FILE_NAMELEN		260
#define FILE_LINELEN		500

enum _FILE_STATUS
{
	FILE_EXIST_NO		= 0,		// Exist status no (does not exist)
	FILE_EXIST_YES		= 1,		// Exist status yes (does exist)

	FILE_STATUS_OK			= 2,	// Return status ok
	FILE_STATUS_EOF			= 3,	// Return status for end-of-file
	FILE_STATUS_READERROR	= 4,	// Return status for read error
	FILE_STATUS_WRITEERROR	= 5,	// Return status for write error
	FILE_STATUS_NOREAD		= 6,	// Return status for no-read (file not opened in read mode)
	FILE_STATUS_NOWRITE		= 7,	// Return status for no-write (file not opened in write mode)

	FILE_POS_BEGIN		= -1,		// Position at beginning of file
	FILE_POS_END		= -2,		// Position at end of file
};
typedef int FileStatus;

enum _FILE_FLAGS
{
	FILE_NOTOPEN		= 0x001, // File not opened yet
	FILE_RANDOM			= 0x002, // Open file in random access mode
	FILE_SEQUENTIAL		= 0x004, // Open file in sequential mode
	FILE_READ			= 0x008, // Allow reading of file
	FILE_WRITE			= 0x010, // Allow writing of file (by overwrite)
	FILE_APPEND			= 0x020, // Allow writing of file (by append)
	FILE_AUTOCREATE		= 0x040, // Allow auto-creation of file if it does not exist
	FILE_LEAVEEOF		= 0x080, // Leave EOF marker when appending
	FILE_RAWDATA		= 0x100, // Read and write unaltered bytes of binary data, use if you encounter problems loading saved data sequentially
};
typedef int FileFlags;


class File
{
public:
	File ();								// Constructor (no file given)
	File (char *fname, FileFlags fmodes);	// Constructor (open file)
	~File ();								// Destructor (close file)

	int Exist (char *fname);					// Exist file (check for file existence) (returns FILE_EXIST_NO if not)
	int Open (char *fname, FileFlags fmodes);	// Open file (using given modes)
	void Close ();							// Close file
	void Delete (char *fname);					// Permanently deletes file (be careful!)
	
	int ReadC (int num, char *buf);			// Read many bytes (chars)
	int WriteC (int num, char *buf);			// Write many bytes
	int Read (int num, unsigned char *buf);				// Read many bytes (ints)
	int Write (int num, unsigned char *buf);			// Write many bytes

	int Read (unsigned char &c);						// Read one byte
	int Write (unsigned char c);						// Write one byte

	int Read2 (unsigned short &c);						// Read two bytes
	int Write2 (unsigned short c);						// Write two bytes

	int Read4 (unsigned long &c);						// Read four bytes
	int Write4 (unsigned long c);						// Write four bytes
	
	int ReadLine (char *buf);
	int WriteLine (char *buf);

	int WriteS (char * str);				// Writes a string

	void SetPosition (long pos);		// Set file position (random only)
	long GetPosition ();			// Get file position (all modes)

	int EndOfFile () {return feof(filehandle);}

#ifdef FILE_COUNT_WRITES
	int write_number; // number of chars written total to this File, for debugging
#endif

private:
	FileFlags filemodes;
	char filename[FILE_NAMELEN];
	FILE *filehandle;
	long filepos;

	unsigned char buffer[FILE_BUFFER];
	long buffer_pos;		
};




// Notes:	Correct behavior for various file modes and functions
//			is outlined below
//
// Files may be opened supporting any of the following functions
//		RANDOM					File mode is random access
//		SEQUENTIAL				File mode is sequential access
//		READ					Allow reading of file in given mode
//		WRITE					Allow overwriting of file in given mode
//		APPEND					Allow append of file in given mode
//		AUTOCREATE				Allow automatic creation of file if does not exist
//		LEAVEEOF				Do not remove end-of-file marker for append
// (note: only overwrite and append are mutually exclusive, so only
// one of them can be used at a time. all others can be used together)
//
// Random access
//								File exists				File does not exist
//		read anywhere			read anywhere			error
//		write anywhere			write anywhere			write (may create)
//		read sequential			read sequential			error
//		overwrite sequential	overwrite sequential	write (may create)
//		append sequential		append sequential		write (may create)
//
// Sequential access
//								File exists				File does not exist
//		read anywhere			error					error
//		write anywhere			error					error
//		read sequential			read sequential			error
//		overwrite sequential	overwrite sequential	write (may create)
//		append sequential		append sequential		write (may create)
//
// Summary of interpretation of Visual C++ fopen modes:
//		"b"		RANDOM
//		"t"		SEQUENTIAL
//		"r"		READ
//		"w"		WRITE, AUTOCREATE
//		"a"		APPEND, AUTOCREATE, LEAVEEOF
//		"r+"	READ, WRITE
//		"w+"	READ, WRITE, AUTOCREATE
//		"a+"	APPEND, AUTOCREATE
//
// The implementation of file opening for the File class in the
// context of Visual C++ is determine by combining all the information
// above.


#endif